using System;
using System.Collections.Generic;
using System.Linq;
using Android.App;
using Android.Content;
using Android.Hardware.Usb;
using Android.OS;
using Android.Views;
using Android.Widget;
using Master.AndroidCode.API;
using Master.Core;

namespace Master.AndroidCode.GUI
{
    [Activity(Label = "Aalberg Audio App", MainLauncher = true, Icon = "@drawable/icon")]
    public class MainGui : ListActivity
    {
        bool isBound = false;
        GuitarControllerService.GuitarControllerServiceBinder _binder;
        GuitarControllerServiceConnection _guitarControllerServiceConnection;
        Intent _guitarControllerServiceIntent;

        private List<IPedal> _devices;
        protected override void OnCreate(Bundle bundle)
        {
            base.OnCreate(bundle);
            _guitarControllerServiceIntent = new Intent("com.aalbergAudio.GuitarController");
            SetContentView(Resource.Layout.main);
        }

        private void CreateInterface()
        {
            ListAdapter = new ArrayAdapter<IPedal>(this, Android.Resource.Layout.SimpleListItem1,
                   _binder.GetGuitarControllerService().GetConnectedDevices());
            _devices = _binder.GetGuitarControllerService().GetConnectedDevices();
            FindViewById<Button>(Resource.Id.TestButton).Click += OnClick;
            FindViewById<Button>(Resource.Id.SavesButton).Click += OpenSaves;
        }

        private void OpenSaves(object sender, EventArgs e)
        {
            var SavesMenu = (new AlertDialog.Builder(this))
                .SetTitle("Saved profiles")
                .SetItems(_binder.GetGuitarControllerService().GetSavedProfiles().Keys.ToArray(), (o, args) => 
                {
                    Console.WriteLine(args.Which.ToString());
                })
                .Create();
        }

        private void ListItemClicked(object sender, DialogClickEventArgs e)
        {
           Console.WriteLine("Test: " + e.ToString());
        }

        private void OnClick(object sender, EventArgs eventArgs)
        {
            _binder.GetGuitarControllerService().SaveProfile("test", _devices);
        }

        protected override void OnListItemClick(ListView l, View v, int position, long id)
        {
            var t = _devices[position];
            var selectedDevice = new Intent(this, typeof(SelectedDeviceActivity));
            selectedDevice.PutExtra("DeviceID", t.Id);
            StartActivity(selectedDevice);
        }
        protected override void OnStart()
        {
            base.OnStart();
            _guitarControllerServiceConnection = new GuitarControllerServiceConnection(this);
            var res = BindService(_guitarControllerServiceIntent, _guitarControllerServiceConnection, Bind.AutoCreate);
        
        }

        protected override void OnStop()
        {
            base.OnStop();
            if (isBound)
            {
                UnbindService(_guitarControllerServiceConnection);
                isBound = false;
            }
        }

        class GuitarControllerServiceConnection : Java.Lang.Object, IServiceConnection
        {
            private readonly MainGui _activity;

            public GuitarControllerServiceConnection(MainGui activity)
            {
                _activity = activity;
            }

            public void OnServiceConnected(ComponentName name, IBinder service)
            {
                var guitarControllerServiceBinder = service as GuitarControllerService.GuitarControllerServiceBinder;
                if (guitarControllerServiceBinder != null)
                {
                    var binder = (GuitarControllerService.GuitarControllerServiceBinder)service;
                    _activity._binder = binder;
                    _activity.isBound = true;
                }
                _activity.CreateInterface();
            }

            public void OnServiceDisconnected(ComponentName name)
            {
                _activity.isBound = false;
            }
        }
    }

    
}