﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Master.Core
{
	internal interface IController{
        string SaveName { get; set; }
		List<string> GetConnectedDeviceNames ();
        List<string> GetConnectedDeviceIds();
        List<IPedal> GetConnectedDevices();

        IPedal GetDeviceByName(string name);
        IPedal GetDeviceById(string id);
	    void UpdateOption(string id, string option, int value);
	    bool IsConnected();
        void SendFirmwareToDevice(string readToEnd);
	    bool CheckForUpdates();
	}

    internal abstract class AbstractController : IController
    {
        public abstract string SaveName { get; set; }

        public List<string> GetConnectedDeviceNames()
        {
            CheckForUpdates();
            return Pedals.Select(d => d.Name).ToList();
        }

        public List<string> GetConnectedDeviceIds()
        {
            CheckForUpdates();
            return Pedals.Select(d => d.Id).ToList();
        }

        public IPedal GetDeviceByName(String name)
        {
            CheckForUpdates();
            return Pedals.FirstOrDefault(d => d.Name == name);
        }

        public IPedal GetDeviceById(string id)
        {
            CheckForUpdates();
            return Pedals.FirstOrDefault(d => d.Id == id);
        }

        protected List<IPedal> Pedals;
        public List<IPedal> GetConnectedDevices()
        {
            var result = GetConnectedDeviceString();
            var pedals = result.Split(';');
            Pedals = new List<IPedal>();
            for(var i = 0; i<pedals.Length-1;i++)
            {
                Pedals.Add(new Pedal(pedals[i], this));
            }
            return Pedals;
        }

        protected abstract string GetConnectedDeviceString();
        public abstract void UpdateOption(string id, string option, int value);
        public abstract bool IsConnected();
        public abstract void SendFirmwareToDevice(string readToEnd);

        public bool CheckForUpdates()
        {
            var updated = GetConnectedDevices();
            //if all pedals are equal to the old ones
            if (
                !updated.Any(
                    pedal =>
                        Pedals.Where(oldPedal => pedal.Id == oldPedal.Id)
                            .TakeWhile(oldPedal => !oldPedal.Equals(pedal))
                            .Any())) return false;
            Pedals = updated;
            return true;
        }
    }
}