using System;
using System.Collections.Generic;
using System.IO;
using System.Xml.Serialization;
using System.Linq;
using Android.Bluetooth;
using Android.InputMethodServices;

namespace Master.Core
{
    public class StorageManager
    {


        public static bool Save(List<IPedal> deviceList, string saveName)
        {
            var path = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            SerializableDictionary<String, List<Pedal>> oldSaves;

            var filename = Path.Combine(path, "saves");
            if (!File.Exists(filename)) oldSaves = new SerializableDictionary<String, List<Pedal>> { { saveName, deviceList.ConvertAll(o => (Pedal)o) } };
            else
            {
                using (var streamReader = new StreamReader(filename))
                {
                    var serializer = new XmlSerializer(typeof(SerializableDictionary<String, List<Pedal>>));
                    oldSaves = serializer.Deserialize(streamReader) as SerializableDictionary<String, List<Pedal>>;
                    if (oldSaves != null) oldSaves[saveName] = deviceList.ConvertAll(o => (Pedal)o);
                }
            }

    try
            {
                using (var writer = new StreamWriter(filename, false))
                {
                    var serializer = new XmlSerializer(typeof(SerializableDictionary<String, List<Pedal>>));
                    serializer.Serialize(writer, oldSaves);
                }
            }
            catch (IOException e)
            {
                return false;
            }
            return true;
        }

        public static Dictionary<string, List<IPedal>> Load()
        {
            var path = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            var filename = Path.Combine(path, "saves");
            if (!File.Exists(filename)) return new SerializableDictionary<string, List<IPedal>>();
            using (var streamReader = new StreamReader(filename))
            {
                var serializer = new XmlSerializer(typeof(SerializableDictionary<String, List<Pedal>>));
                var temp = serializer.Deserialize(streamReader) as SerializableDictionary<String, List<Pedal>>;
                var returnArray = new SerializableDictionary<string, List<IPedal>>();
                foreach (var a in temp)
                {
                    returnArray[a.Key] = a.Value.ToList< IPedal >();
                }
                return returnArray;
            }
        }
            
    }
}